#include "shader.h"

char *shader_read_file(const char *file_path) {
  FILE *fp;
  char *buf;
  long file_size;
  fp = fopen(file_path, "r");
  if (fp == NULL) {
    fprintf(stderr, "Failed to open file: %s\n", file_path);
    return (char *)NULL;
  }
  fseek(fp, 0, SEEK_END);
  file_size = ftell(fp);
  fseek(fp, 0, SEEK_SET);

  buf = (char *)malloc(file_size + 1);
  if (buf == NULL) {
    fprintf(stderr, "Failed to allocate memory\n");
    fclose(fp);
    return (char *)NULL;
  }
  fread(buf, file_size, 1, fp);
  buf[file_size] = '\0';
  fclose(fp);
  return buf;
}

GLuint load_shader(const char *vfp, const char *ffp) {
  GLuint vertex_id = glCreateShader(GL_VERTEX_SHADER);
  GLuint fragment_id = glCreateShader(GL_FRAGMENT_SHADER);
  char *err = (char *)malloc(sizeof(char *));
  char *v_source = shader_read_file(vfp);
  char *f_source = shader_read_file(ffp);

  if (v_source == NULL) {
    fprintf(stderr, "An error occurred\n");
    return (GLuint)0;
  }
  if (f_source == NULL) {
    fprintf(stderr, "An error occurred\n");
    return (GLuint)0;
  }

  GLint result = GL_FALSE;
  int info_log_len;

  printf("Compiling shader: %s\n", vfp);
  char const *vc_source = v_source;
  glShaderSource(vertex_id, 1, &vc_source, NULL);
  glCompileShader(vertex_id);
  glGetShaderiv(vertex_id, GL_COMPILE_STATUS, &result);
  glGetShaderiv(vertex_id, GL_INFO_LOG_LENGTH, &info_log_len);
  if (info_log_len > 0) {
    glGetShaderInfoLog(vertex_id, info_log_len, NULL, err);
    puts(err);
  }

  printf("Compiling shader: %s\n", ffp);
  char const *fc_source = f_source;

  glShaderSource(fragment_id, 1, &fc_source, NULL);
  glCompileShader(fragment_id);
  glGetShaderiv(fragment_id, GL_COMPILE_STATUS, &result);
  glGetShaderiv(vertex_id, GL_INFO_LOG_LENGTH, &info_log_len);
  if (info_log_len > 0) {
    glGetShaderInfoLog(fragment_id, info_log_len, NULL, err);
    puts(err);
  }
  free(v_source);
  free(f_source);

  puts("Linking program");
  GLuint pid = glCreateProgram();
  glAttachShader(pid, vertex_id);
  glAttachShader(pid, fragment_id);
  glLinkProgram(pid);

  glGetProgramiv(pid, GL_LINK_STATUS, &result);
  glGetProgramiv(pid, GL_INFO_LOG_LENGTH, &info_log_len);
  if (info_log_len > 0) {
    glGetShaderInfoLog(pid, info_log_len, NULL, err);
    puts(err);
  }
  free(err);

  glDetachShader(pid, vertex_id);
  glDetachShader(pid, fragment_id);
  glDeleteShader(vertex_id);
  glDeleteShader(fragment_id);

  return pid;
}
